@@ -118,7 +118,8 @@ GetOptions(
   "v|verbose"         => \$verbose,
 ) or die "bad args";
 Getopt::Long::Configure("default");
-say "> ARGV PASSED THROUGH: ",join(",",map{ "'${_}'" } @ARGV) if $debug;
+say "> ARGV PASSED THROUGH: ",join(",",map{ "'${_}'" } @ARGV)
+  if @ARGV && $debug;
 
 $dvs{debug}   = $debug   if defined($debug);
 $dvs{verbose} = $verbose if defined($verbose);
@@ -235,10 +236,10 @@ sub string_to_tempfile($@) {
 # override that.   I'm forcing LC_ALL=C so things like date and number
 # formats will be predictable for testing.
 #
-# This is usually enclosed in Tiny::Capture::capture { ... }
+# This is usually enclosed in Capture::Tiny::capture { ... }
 #
 #    ==> IMPORTANT: Be sure STDOUT/ERR has :encoding(...) set beforehand
-#        because Tiny::Capture will decode captured output the same way.
+#        because Capture::Tiny will decode captured output the same way.
 #        Otherwise wide chars will be corrupted
 #
 #
@@ -574,8 +575,8 @@ sub mycheckeq_literal($$$) {
                           .($vposn > 0 ? "(line ".($vposn+1).")\n" : "\n")
         ." at line ", (caller(0))[2]."\n"
        ) ;
-  #goto &Carp::confess;
-  Carp::confess(@_);
+  goto &Carp::confess;
+  #Carp::confess(@_);
 }
 sub expect1($$) {
   @_ = ("", @_);
